unit mainform;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, OpenGLContext, Forms, Controls, Graphics,
  Dialogs, gl, glu, glext;

type

  { TForm1 }

  TForm1 = class(TForm)
    OpenGLControl1: TOpenGLControl;
    procedure OpenGLControl1Paint(Sender: TObject);
  private

  public
    prog : gluint;
    vshd : gluint;
    fshd : gluint;
    procedure InitShaders;
    function GetVertexShaderCode: string;
    function GetFragmentShaderCode: string;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}



{ TForm1 }

procedure TForm1.OpenGLControl1Paint(Sender: TObject);
begin
  glClearColor(0, 1, 0.5, 1);
  glClear(GL_COLOR_BUFFER_BIT);

  if prog=0 then InitShaders;

  glUseProgram(prog);
  glBegin(GL_TRIANGLES);
      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(-1.0, 1.0, 1.0);
      glVertex3f(1.0, -1.0, 1.0);
  glEnd();
  glUseProgram(0);

  OpenGLControl1.SwapBuffers;
end;

procedure DumpCompiled(sh: gluint);
var
  st: gluint;
  log : string;
  len : glsizei;
begin
  glGetShaderiv(sh, GL_COMPILE_STATUS, @st);
  if st=0 then begin
    SetLength(log, 2048);
    len:=length(log);
    glGetShaderInfoLog(sh, len, @len, @log[1]);
    SetLength(log, len);
  end;
  SetLength(log, 2048);
  len:=length(log);
  glGetShaderSource(sh, len, @len, @log[1]);
  SetLength(log, len);

end;

procedure TForm1.InitShaders;
var
  src : string;
  len : glint;
  log : string;
  cmp : PChar;
  st  : gluint;
begin
  //Load_GL_ARB_shader_objects;
  //Load_GL_ARB_vertex_shader;
  Load_GL_version_2_0;

  prog := glCreateProgram;
  src := GetVertexShaderCode;
  if src<>'' then begin
    vshd := glCreateShader(GL_VERTEX_SHADER);
    len:=length(src);
    cmp:=@src[1];
    glShaderSource(vshd, 1, @cmp, @len);
    glCompileShader(vshd);
    DumpCompiled(vshd);
  end;

  src := GetFragmentShaderCode;
  if src<>'' then begin
    fshd := glCreateShader(GL_FRAGMENT_SHADER);
    len:=length(src);
    cmp:=@src[1];
    glShaderSource(fshd, 1, @cmp, @len);
    glCompileShader(fshd);
    DumpCompiled(fshd);
  end;

  glAttachShader(prog, vshd);
  glAttachShader(prog, fshd);

  glLinkProgram(prog);
  //glGetProgramiv(prog, GL_LINK_STATUS, @st);

  //SetLength(log, 1024);
  //len:=length(log);
  //glGetProgramInfoLog(prog, len, @len, @log[1]);
  //SetLength(log, len);
  //writeln(log);

  //glDeleteObjectARB(vshd);
  //glDeleteObjectARB(fshd);

end;

function TForm1.GetVertexShaderCode: string;
begin
  Result:=
'#version 330 compatibility'#10+
'void main() '#10+
'{'#13+
'    // Transforming The Vertex'#10+
'    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;'#10+
'}';
  Result:=
'#version 330 compatibility'#10+
'void main() { gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex; }'
end;

function TForm1.GetFragmentShaderCode: string;
begin
  Result:=
'#version 330 compatibility'#10+
'void main() '#10+
'{'#10+
'    // Setting Each Pixel To Red'#10+
'    gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);'#10+
'}';
end;


end.

